@@ -10,6 +10,7 @@ from einops import rearrange
 from dalle_pytorch import distributed_utils
 from dalle_pytorch.vae import OpenAIDiscreteVAE, VQGanVAE
 from dalle_pytorch.transformer import Transformer, DivideMax
+from dalle_pytorch.attention import stable_softmax
 
 # helpers
 
@@ -324,6 +325,7 @@ class DALLE(nn.Module):
         attn_types = None,
         loss_img_weight = 7,
         stable = False,
+        sandwich_norm = False,
         shift_tokens = True,
         rotary_emb = True
     ):
@@ -371,6 +373,7 @@ class DALLE(nn.Module):
             image_fmap_size = image_fmap_size,
             sparse_attn = sparse_attn,
             stable = stable,
+            sandwich_norm = sandwich_norm,
             shift_tokens = shift_tokens,
             rotary_emb = rotary_emb
         )
@@ -439,7 +442,7 @@ class DALLE(nn.Module):
             logits = logits[:, -1, :]
 
             filtered_logits = top_k(logits, thres = filter_thres)
-            probs = F.softmax(filtered_logits / temperature, dim = -1)
+            probs = stable_softmax(filtered_logits / temperature, dim = -1)
             sample = torch.multinomial(probs, 1)
  
             text_tokens = torch.cat((text_tokens, sample), dim=-1)
@@ -555,6 +558,10 @@ class DALLE(nn.Module):
             seq_len -= 1
             tokens = tokens[:, :-1]
 
+        if self.stable:
+            alpha = 0.1
+            tokens = tokens * alpha + tokens.detach() * (1 - alpha)
+
         out = self.transformer(tokens)
 
         if self.stable:
